<?php
/**
 * @version    $Id$
 * @package    JSN Extension Framework 2
 * @author     JoomlaShine Team <support@joomlashine.com>
 * @copyright  Copyright (C) 2012 JoomlaShine.com. All Rights Reserved.
 * @license    GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
 *
 * Websites: http://www.joomlashine.com
 * Technical Support:  Feedback - http://www.joomlashine.com/contact-us/get-support.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Base class for proxying an Ajax request.
 *
 * @package  JSN Extension Framework 2
 * @since    1.0.0
 */
class JsnExtFwAjaxProxy extends JsnExtFwAjax
{
	/**
	 * Define target component.
	 *
	 * @var  string
	 */
	protected $option = '';

	/**
	 * Define additional query parameters.
	 *
	 * @var  string
	 */
	protected $query_params = '';

	public function indexAction()
	{
		// Backup the current error reporting level.
		$error_reporting = ini_get('error_reporting');

		// Then, disable error reporting to prevent unwanted notice and warning from being shown.
		error_reporting(0);

		// Get request method.
		$method = strtolower($_SERVER['REQUEST_METHOD']);

		// Get request URL.
		$link = JUri::getInstance()->toString();

		// Get request headers.
		$headers = getallheaders();

		// Get posted data.
		$data = JFactory::getApplication()->input->post->getArray();

		// Prepare to proxy the request.
		$link = preg_replace('/[?&]group=[^&]+/', '', $link);
		$link = preg_replace('/[?&]plugin=[^&]+/', '', $link);
		$link = preg_replace('/[?&]format=[^&]+/', '', $link);
		$link = preg_replace('/[?&]context=[^&]+/', '', $link);

		if (array_key_exists('option', $data))
		{
			$data['option'] = $this->option;
		}
		elseif (strpos($link, '?option=') !== false || strpos($link, '&option=') !== false)
		{
			$link = preg_replace('/([?&])option=[^&]+/', "\\1option={$this->option}", $link);
		}

		if (!empty($this->query_params))
		{
			$link .= (strpos($link, '?') === false ? '?' : '&') . ltrim($this->query_params, '&');
		}

		// Proxy the request.
		$http = new JHttp();

		if (in_array($method, array('delete', 'get', 'head', 'options', 'trace')))
		{
			$http = call_user_func(array($http, $method), $link, $headers);
		}
		elseif (in_array($method, array('patch', 'post', 'put')))
		{
			$http = call_user_func(array($http, $method), $link, $data, $headers);
		}

		$this->setResponse($http->body);

		// Restore error reporting level.
		error_reporting($error_reporting);
	}

	/**
	 * Process and set response content.
	 *
	 * @param   mixed  $content  Content will be sent to client.
	 *
	 * @return  void
	 */
	protected function setResponse($content)
	{
		// Replace target link with proxy link.
		$pattern = "/(action|href)=(['\"])([^'\"]*index\.php\?option={$this->option}[^'\"]*)(['\"])/";

		if (preg_match_all($pattern, $content, $matches, PREG_SET_ORDER))
		{
			$base = str_replace('&', '&amp;', $this->baseUrl);

			foreach ($matches as $match)
			{
				// Clear token from link.
				$link = preg_replace('/&(amp;)?[a-z0-9]{32}=1/', '', $match[3]);

				// Generate proxy link from target link.
				$link = str_replace("index.php?option={$this->option}", $base, $link);

				// Replace target link with proxy link.
				$content = str_replace($match[0], "{$match[1]}={$match[2]}{$link}{$match[4]}", $content);
			}
		}

		parent::setResponse($content);
	}
}
