<?php
/**
 * @version    $Id$
 * @package    JSN_PageBuilder_4
 * @author     JoomlaShine Team <support@joomlashine.com>
 * @copyright  Copyright (C) 2012 JoomlaShine.com. All Rights Reserved.
 * @license    GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
 *
 * Websites: http://www.joomlashine.com
 * Technical Support:  Feedback - http://www.joomlashine.com/contact-us/get-support.html
 */

// No direct access to this file.
defined('_JEXEC') or die('Restricted access');

/**
 * Handle Ajax requests.
 *
 * @package  JSN_PageBuilder_4
 * @since    1.0.0
 */
class JsnExtFwAjaxPb4Editor extends JsnExtFwAjax
{
	/**
	 * Method to get the user agent of current browser.
	 *
	 * @return  void
	 */
	public function getUserAgentAction()
	{
		$this->setResponse($_SERVER['HTTP_USER_AGENT']);
	}

	/**
	 * Method to render sandbox for the editor app.
	 *
	 * @return  void
	 */
	public function renderSandboxAction()
	{
		// Load required assets.
		JsnExtFwAssets::loadJsnComponents();

		// Init event tracking.
		//JSNPageBuilder4Helper::initEventTracking();

		// Init edition manager.
		JSNPageBuilder4Helper::initEditionManager();

		// Get JSN PageBuilder 4 parameters.
		$config = JsnExtFwHelper::getSettings('com_pagebuilder4');
		$liveChatHtml = '';

		if ($config['live_chat_popup'])
		{
			try
			{
				$liveChatHtml = JsnExtFwHttp::get(
					JsnExtFwHelper::getConstant('LIVE_CHAT_URL', 'com_pagebuilder4'),
					24 * 60 * 60
				);
			}
			catch (Exception $e)
			{
				error_log($e->getMessage());
			}
		}

		$this->render('index', compact('liveChatHtml'), dirname(__FILE__) . '/tmpl');
	}

	/**
	 * Method for retrieving dynamic content.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=get-content&<FORM_TOKEN>=1
	 *
	 * Use the following query parameters to filter items list:
	 * - type: Supported type are:
	 *         + articles: Get Joomla articles.
	 *         + authors: Get Joomla authors.
	 *         + categories: Get Joomla categories.
	 *         + modules: Get Joomla modules.
	 *         + tags: Get Joomla tags.
	 *         + k2_authors: Get K2 authors.
	 *         + k2_categories: Get K2 categories.
	 *         + k2_items: Get K2 items.
	 *         + k2_tags: Get K2 tags.
	 *         + easyblog_authors: Get EasyBlog authors.
	 *         + easyblog_categories: Get EasyBlog categories.
	 *         + easyblog_posts: Get EasyBlog items.
	 *         + easyblog_tags: Get EasyBlog tags.
	 *         + jsn_uniform_forms: Get JSN UniForm form list.
	 * - limit: Total number of items to retrieve.
	 * - start: Start retrieving items from this index.
	 * - order_by: Visit the items list screen of the appropriated type to get available options.
	 * - order_dir: Either `asc` or `desc`.
	 * - filter_author: Filter items list by an author ID or list of author ID separated by comma.
	 * - filter_category: Filter items list by a category ID or list of category ID separated by comma.
	 * - filter_tag: Filter items list by a tag ID or list of tag ID separated by comma.
	 * - filter_search: Filter items list by a keyword.
	 *
	 * Please note that `filter_author`, `filter_category` and `filter_tag` are supported when retrieving
	 * content of type `articles`, `k2_items` and `easyblog_posts` only.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function getContentAction()
	{
		// Get request parameters.
		$start = $this->input->getInt('start', 0);
		$limit = $this->input->getInt('limit', 9999);
		$type = $this->input->getString('type', 'articles');
		$order_by = $this->input->getString('order_by', '');
		$order_dir = $this->input->getString('order_dir', '');

		// Generate filters array from request parameters.
		$filters = array();

		if (($option = $this->input->getString('filter_author', '')) !== '')
		{
			$filters['author_id'] = array_filter(explode(',', $option), 'intval');
		}

		if (($option = $this->input->getString('filter_category', '')) !== '')
		{
			$filters['category_id'] = array_filter(explode(',', $option), 'intval');
		}

		if (($option = $this->input->getString('filter_tag', '')) !== '')
		{
			$filters['tag'] = array_filter(explode(',', $option), 'intval');
		}

		if (($option = $this->input->getString('filter_search', '')) !== '')
		{
			$filters['search'] = trim($option);
		}

		// Retrieve dynamic content.
		$this->setResponse(
			JSNPageBuilder4Helper::getContentItems(
				$type, $limit, $start, trim("{$order_by} {$order_dir}"), $filters
			)
		);
	}

	/**
	 * Method to save editor section.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=save-editor-section&<FORM_TOKEN>=1
	 *
	 * Post the following parameters in request body:
	 *
	 * - name: Section name.
	 * - image: Section thumbnail.
	 * - data: Section data.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function saveEditorSectionAction()
	{
		// Only allow POST method.
		if (strtoupper($_SERVER['REQUEST_METHOD']) !== 'POST')
		{
			throw new Exception(JText::_('JSN_PAGEBUILDER4_INVALID_REQUEST_METHOD'));
		}

		// Get request parameters.
		$name = $this->input->post->getString('name');
		$image = $this->input->post->get('image', '', 'raw');
		$data = $this->input->post->get('data', '', 'raw');

		// Save editor section.
		return $this->setResponse(JSNPageBuilder4Helper::saveEditorSection($name, $image, $data));
	}

	/**
	 * Method to rename editor section.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=rename-editor-section&<FORM_TOKEN>=1
	 *
	 * Post the following parameters in request body:
	 *
	 * - id: Record ID of the section saved in database.
	 * - name: New section name.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function renameEditorSectionAction()
	{
		// Only allow POST method.
		if (strtoupper($_SERVER['REQUEST_METHOD']) !== 'POST')
		{
			throw new Exception(JText::_('JSN_PAGEBUILDER4_INVALID_REQUEST_METHOD'));
		}

		// Get request parameters.
		$id = $this->input->post->getInt('id');
		$name = $this->input->post->getString('name');

		// Rename editor section.
		return $this->setResponse(JSNPageBuilder4Helper::renameEditorSection($id, $name));
	}

	/**
	 * Method to remove editor section.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=remove-editor-section&<FORM_TOKEN>=1
	 *
	 * The following query parameter is required:
	 *
	 * - id: Record ID of the section saved in database.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function removeEditorSectionAction()
	{
		// Get request parameters.
		$id = $this->input->getInt('id');

		// Remove editor section.
		return $this->setResponse(JSNPageBuilder4Helper::removeEditorSection($id));
	}

	/**
	 * Method to get saved editor sections.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=get-editor-sections&<FORM_TOKEN>=1
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function getEditorSectionsAction()
	{
		// Get saved editor sections.
		return $this->setResponse(JSNPageBuilder4Helper::getEditorSections());
	}

	/**
	 * Method to save page data.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=save-page-data&<FORM_TOKEN>=1
	 *
	 * Post the following parameters in request body:
	 *
	 * - extension: Extension of the content item that is being saved.
	 *              E.g. com_content when saving a Joomla article, com_k2 if saving a K2 item, etc.
	 * - hash: Hash of the page which data is being saved.
	 * - data: Page data.
	 * - html: Generated HTML markups.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function savePageDataAction()
	{
		// Only allow POST method.
		if (strtoupper($_SERVER['REQUEST_METHOD']) !== 'POST')
		{
			throw new Exception(JText::_('JSN_PAGEBUILDER4_INVALID_REQUEST_METHOD'));
		}

		// Get request parameters.
		$extension = $this->input->post->getCmd('extension');
		$hash = $this->input->post->getString('hash');
		$data = $this->input->post->get('data', '', 'raw');
		$html = $this->input->post->get('html', '', 'raw');

		// Save page data.
		return $this->setResponse(
			JSNPageBuilder4Helper::savePageData($extension, $hash, $data, $html)
		);
	}

	/**
	 * Method to get page data.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=get-page-data&<FORM_TOKEN>=1
	 *
	 * The following query parameter is required:
	 *
	 * - hash: Hash of page to get data for. If a number is provided,
	 *         database record with ID equivalent to that number will be returned.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function getPageDataAction()
	{
		// Get request parameters.
		$hash = $this->input->getString('hash');

		// Get page data.
		return $this->setResponse(JSNPageBuilder4Helper::getPageData($hash));
	}

	/**
	 * Method to save page revision.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=save-page-revision&<FORM_TOKEN>=1
	 *
	 * Post the following parameters in request body:
	 *
	 * - hash: Hash of the page which revision is being saved.
	 * - type: Revision type.
	 * - data: Revision data.
	 * - html: Generated HTML markups.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function savePageRevisionAction()
	{
		// Only allow POST method.
		if (strtoupper($_SERVER['REQUEST_METHOD']) !== 'POST')
		{
			throw new Exception(JText::_('JSN_PAGEBUILDER4_INVALID_REQUEST_METHOD'));
		}

		// Get request parameters.
		$hash = $this->input->post->getString('hash');
		$type = $this->input->post->getString('type');
		$data = $this->input->post->get('data', '', 'raw');
		$html = $this->input->post->get('html', '', 'raw');

		// Save page revision.
		return $this->setResponse(JSNPageBuilder4Helper::savePageRevision($hash, $data, $html, $type));
	}

	/**
	 * Method to remove page revision.
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=remove-page-revision&<FORM_TOKEN>=1
	 *
	 * The following query parameter is required:
	 *
	 * - id: Record ID of the page revision saved in database.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function removePageRevisionAction()
	{
		// Get request parameters.
		$id = $this->input->getInt('id');

		// Remove page revision.
		return $this->setResponse(JSNPageBuilder4Helper::removePageRevision($id));
	}

	/**
	 * Method to get page revision(s).
	 *
	 * Ajax URL: index.php?option=com_ajax&format=json&plugin=pagebuilder4&action=get-page-revisions&<FORM_TOKEN>=1
	 *
	 * The following query parameter is required:
	 *
	 * - hash: Hash of page to get data for. If a number is provided,
	 *         database record with ID equivalent to that number will
	 *         be returned. Otherwise, all database record associated
	 *         with the provided page hash will be returned.
	 *
	 * Remember to replace <FORM_TOKEN> in the Ajax URL above with the real form token. Form token can be
	 * retrieved by calling the static method `getFormToken` of the class `JSession`.
	 *
	 * @return  void
	 */
	public function getPageRevisionsAction()
	{
		// Get request parameters.
		$hash = $this->input->getString('hash');

		// Get page revision.
		return $this->setResponse(JSNPageBuilder4Helper::getPageRevisions($hash));
	}
}
